@@ -1,8 +1,8 @@
 swN2 <- function(pressure, sigmaTheta=NULL, derivs, ...) # BUG: think more about best density measure
 {
     if (inherits(pressure, "ctd")) {
-        sigmaTheta <- swSigmaTheta(pressure$data$salinity, pressure$data$temperature, pressure$data$pressure)
-        pressure <- pressure$data$pressure # over-writes pressure
+        sigmaTheta <- swSigmaTheta(pressure@data$salinity, pressure@data$temperature, pressure@data$pressure)
+        pressure <- pressure@data$pressure # over-writes pressure
     }
     ok <- !is.na(pressure) & !is.na(sigmaTheta)
     if (missing(derivs)) {
@@ -106,8 +106,8 @@ swTFreeze <- function(salinity, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     (-.0575+1.710523e-3*sqrt(abs(salinity))-2.154996e-4*salinity)*salinity-7.53e-4*pressure
 }
@@ -117,9 +117,9 @@ swAlpha <- function(salinity, temperature=NULL, pressure=NULL, isTheta = FALSE)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -134,9 +134,9 @@ swAlphaOverBeta <- function(salinity, temperature=NULL, pressure=NULL, isTheta =
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -173,9 +173,9 @@ swBeta <- function(salinity, temperature=NULL, pressure=NULL, isTheta = FALSE)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -210,9 +210,9 @@ swBeta <- function(salinity, temperature=NULL, pressure=NULL, isTheta = FALSE)
 swConductivity <- function (salinity, temperature=NULL, pressure=NULL)
 {
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     return(0.57057 * (1 + temperature * (0.003 - 1.025e-05 * temperature) + 0.000653 * pressure - 0.00029 * salinity))
 }
@@ -220,8 +220,8 @@ swConductivity <- function (salinity, temperature=NULL, pressure=NULL)
 swDepth <- function(pressure, latitude=45, degrees=TRUE)
 {
     if (inherits(pressure, "ctd")) {
-        latitude <- abs(pressure$metadata$latitude)
-        pressure <- pressure$data$pressure # over-writes pressure
+        latitude <- abs(pressure@metadata$latitude)
+        pressure <- pressure@data$pressure # over-writes pressure
     }
     if (degrees) latitude <- latitude * 0.0174532925199433
     x <- sin(latitude)^2
@@ -231,6 +231,11 @@ swDepth <- function(pressure, latitude=45, degrees=TRUE)
 
 swZ <- function(pressure, latitude=45, degrees=TRUE)
 {
+    if (inherits(pressure, "ctd")) {
+        keep <- pressure
+        pressure <- keep[["pressure"]]
+        latitude <- keep[["latitude"]]
+    }
     -swDepth(pressure=pressure, latitude=latitude, degrees=degrees)
 }
 
@@ -238,27 +243,27 @@ swDynamicHeight <- function(x, referencePressure=2000)
 {
     height <- function(ctd, referencePressure)
     {
-        if (sum(!is.na(ctd$data$pressure)) < 2) return(NA) # cannot integrate then
-        g <- if (is.na(ctd$metadata$latitude)) 9.8 else gravity(ctd$metadata$latitude)
-        np <- length(ctd$data$pressure)
+        if (sum(!is.na(ctd@data$pressure)) < 2) return(NA) # cannot integrate then
+        g <- if (is.na(ctd@metadata$latitude)) 9.8 else gravity(ctd@metadata$latitude)
+        np <- length(ctd@data$pressure)
         rho <- swRho(ctd)
         if (sum(!is.na(rho)) < 2) return(NA)
         ## 1e4 converts decibar to Pa
-        dzdp <- ((1/rho - 1/swRho(rep(35,np),rep(0,np),ctd$data$pressure))/g)*1e4
+        dzdp <- ((1/rho - 1/swRho(rep(35,np),rep(0,np),ctd@data$pressure))/g)*1e4
 ##        print(summary(ctd))
-        integrand <- approxfun(ctd$data$pressure, dzdp, rule=2)
+        integrand <- approxfun(ctd@data$pressure, dzdp, rule=2)
         integrate(integrand, 0, referencePressure)$value
     }
     if (inherits(x, "section")) {
-        lon0 <- x$data$station[[1]]$metadata$longitude
-        lat0 <- x$data$station[[1]]$metadata$latitude
-        ns <- length(x$data$station)
+        lon0 <- x@data$station[[1]]@metadata$longitude
+        lat0 <- x@data$station[[1]]@metadata$latitude
+        ns <- length(x@data$station)
         d <- vector("numeric", ns)
         h <- vector("numeric", ns)
         for (i in 1:ns) {               # FIXME: avoid loops
 ##            cat("i=",i,"\n")
-            d[i] <- geodDist(x$data$station[[i]]$metadata$latitude, x$data$station[[i]]$metadata$longitude, lat0, lon0)
-            h[i] <- height(x$data$station[[i]], referencePressure)
+            d[i] <- geodDist(x@data$station[[i]]@metadata$latitude, x@data$station[[i]]@metadata$longitude, lat0, lon0)
+            h[i] <- height(x@data$station[[i]], referencePressure)
         }
         return(list(distance=d, height=h))
     } else if (inherits(x, "ctd")) {
@@ -273,9 +278,9 @@ swLapseRate <- function(salinity, temperature=NULL, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -305,9 +310,9 @@ swRho <- function(salinity, temperature=NULL, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -342,9 +347,9 @@ swSigma <- function(salinity, temperature=NULL, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     swRho(salinity, temperature, pressure) - 1000
 }
@@ -354,9 +359,9 @@ swSigmaT <- function(salinity, temperature=NULL, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     ptop <- rep(0, length(salinity))
     swRho(salinity, temperature, ptop) - 1000
@@ -367,9 +372,9 @@ swSigmaTheta <- function(salinity, temperature=NULL, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     ptop <- rep(0, length(salinity))
     swRho(salinity, swTheta(salinity, temperature, pressure), ptop) - 1000
@@ -380,9 +385,9 @@ swSoundSpeed <- function(salinity, temperature=NULL, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -419,9 +424,9 @@ swSpecificHeat <- function(salinity, temperature=NULL, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <-  salinity$data$temperature
-        pressure <-  salinity$data$pressure
-        salinity <-  salinity$data$salinity # note: this destroys the ctd object
+        temperature <-  salinity@data$temperature
+        pressure <-  salinity@data$pressure
+        salinity <-  salinity@data$salinity # note: this destroys the ctd object
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -453,9 +458,9 @@ swSpice <- function(salinity, temperature=NULL, pressure=NULL)
     if (missing(salinity))
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
-        temperature <- salinity$data$temperature
-        pressure <- salinity$data$pressure
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <- salinity@data$temperature
+        pressure <- salinity@data$pressure
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -491,9 +496,9 @@ swTheta <- function(salinity, temperature=NULL, pressure=NULL, referencePressure
         stop("must provide salinity")
     if (inherits(salinity, "ctd")) {
         tmp <- salinity
-        salinity <- tmp$data$salinity
-        temperature <- tmp$data$temperature
-        pressure <- tmp$data$pressure
+        salinity <- tmp@data$salinity
+        temperature <- tmp@data$temperature
+        pressure <- tmp@data$pressure
     }
     if (is.null(temperature))
         stop("must provide temperature")
@@ -537,8 +542,8 @@ swTheta <- function(salinity, temperature=NULL, pressure=NULL, referencePressure
 swViscosity <- function(salinity, temperature=NULL)
 {
     if (inherits(salinity, "ctd")) {
-        temperature <-  salinity$data$temperature
-        salinity <- salinity$data$salinity # note: this destroys the ctd object
+        temperature <-  salinity@data$temperature
+        salinity <- salinity@data$salinity # note: this destroys the ctd object
     }
     0.001798525 + salinity * (2.634749e-06 - 7.088328e-10 *
            temperature^2 + salinity * (-4.702342e-09 + salinity *
